# The original file is in EngSys-AzSec-LinuxExtensions
DISTRO=
VER=
SSL_CERT_DIR=
SSL_CERT_FILE=

function GetLnxBase {
  case $DISTRO in
    ubuntu | debian | raspbian | cbld | elementary)
      lnxBase="deb"
      ;;
    centos | redhat | fedoraproject | sles | opensuse | oracle | mariner | azurelinux | amazon | rocky | almalinux)
      lnxBase="lsb"
      ;;
    *)
      lnxBase="unsupported"
      ;;
  esac
  echo $lnxBase
}

function SetSSL {
  case $DISTRO in
    ubuntu | debian | raspbian | cbld | elementary)
      SSL_CERT_DIR="/etc/ssl/certs"
      ;;
    centos | redhat | fedoraproject | oracle | mariner | azurelinux | amazon | rocky | almalinux)
      SSL_CERT_FILE="/etc/pki/tls/certs/ca-bundle.crt"
      ;;
    sles | opensuse)
      if [[ $VER == 11* ]] ; then
        SSL_CERT_DIR="/etc/ssl/certs"
        SSL_CERT_FILE="/etc/ssl/certs/mdsd-ca-certs.pem"
      elif [[ $VER == 12* ]] ; then
        SSL_CERT_DIR="/var/lib/ca-certificates/openssl"
      elif [[ $VER == 15* ]] ; then
        SSL_CERT_DIR="/etc/ssl/certs"
      fi
      ;;
  esac
}

detect_installer()
{
    INSTALLER=

    # If DPKG lives here, assume we use that. Otherwise we use RPM.
    check_if_program_in_path dpkg
    if [ $? -eq 0 ]; then
        INSTALLER=DPKG
    else
      check_if_program_in_path rpm
      if [ $? -eq 0 ]; then
        INSTALLER=RPM
      else
        #Exit with code 51 if system is not deb or rpm
        echo "Error: This system does not have supported package manager"
        echo "Supported Sytems: 'DPKG' & 'RPM'"
        exit 1
      fi
    fi
}

# $1 - The name of the package to check as to whether it's installed
check_if_pkg_is_installed()
{
    detect_installer

    if [ "$INSTALLER" = "DPKG" ]; then
        dpkg -s $1 2> /dev/null | grep Status | grep " installed" 1> /dev/null
    else
        rpm -q $1 > /dev/null 2>&1
    fi

    return $?
}

check_if_program_in_path()
{
    # Parameter: name of program to check
    # Returns: 0 if program is in path, non-zero if not
    if [ $# -ne 1 ]; then
        echo "INTERNAL ERROR: Incorrect number of parameters passed to check_if_program_in_path" >&2
        return 1
    fi
    which $1 > /dev/null 2>&1
    return $?
}

check_if_program_exists_on_system()
{
    # Parameters: $1 - name of program to check
    # Returns: 0 if program is in system or installed as a package, 1 if not
    if [ $# -ne 1 ]; then
        echo "INTERNAL ERROR: Incorrect number of parameters passed to check_if_program_exists_on_system" >&2
        return 1
    fi
    local exists=1
    check_if_pkg_is_installed $1
    [ $? -eq 0 ] && exists=0
    check_if_program_in_path $1
    [ $? -eq 0 ] && exists=0
    return $exists
}

# start/stop/restart service
# OpService name [start|stop|restart]
# returns the exit code for the given service command; to check result or ignore without
# the calling script exiting (if -e is set), append "|| true" or "|| ret=$?" to the call
function OpService {
  servicename=$1
  cmd=$2
  if [ -z $servicename ]; then
    echo "error in OpService: service name is not set"
    return 1
  fi
  if [ -z $cmd ]; then
    echo "error in OpService: command is not set"
    return 1
  fi

  # Only run the following commands if this system is booted with systemd.
  # For example, an Ubuntu-based container is not booted with systemd,
  # so even though we detect systemd, running systemctl commands is unnecessary.
  if [ -d /run/systemd/system ]; then
    systemctl $cmd $servicename.service
    exit_code=$?
    if [ "$exit_code" -ne 0 ]; then
      echo "Failed to $cmd $servicename"
      return $exit_code
    fi
  fi
}

function OpServiceIfExist {
  servicename=$1
  cmd=$2
  if [ -z $servicename ]; then
    echo "error in OpServiceIfExist: service name is not set"
    return 1
  fi

  if [ -f /etc/init.d/$servicename ]; then
    OpService $servicename $cmd || true
    return 0
  fi
  
  # when the service not found in init.d check it using package manager
  set +e # this is important to prevent exiting when $? changed
  check_if_program_exists_on_system $servicename
  if [ $? -eq 0 ]; then
      OpService $servicename $cmd
  fi
  set -e # reset it back
}

# determine distro and version
if [ -e /etc/system-release-cpe ]; then
  cpe_version=`awk -F":" '{print $2}' /etc/system-release-cpe`
  if [ $cpe_version = "/o" ]; then
    DISTRO=`awk -F":" '{print $3}' /etc/system-release-cpe`
    VER=`awk -F":" '{print $5}' /etc/system-release-cpe`
  else
    DISTRO=`awk -F":" '{print $4}' /etc/system-release-cpe`
    VER=`awk -F":" '{print $6}' /etc/system-release-cpe`
  fi
elif [ -e /etc/os-release ]; then
  DISTRO=`grep "^ID=" /etc/os-release | awk -F"=" '{print $2}' | sed 's/"//g' | awk -F"-" '{print $1}'`
  VER=`grep "^VERSION_ID=" /etc/os-release | awk -F"=" '{print $2}' | sed 's/"//g'`
fi

LNXBASE=$(GetLnxBase)
SetSSL
